@@ -10,7 +10,10 @@
 extern SEXP big_sd(SEXP X_, SEXP ncore_);
 
 // standardize a matrix of filebacked data
-extern SEXP big_std(SEXP X_, SEXP ncore_);
+extern SEXP big_std(SEXP X_,
+                    SEXP ncore_,
+                    SEXP center_,
+                    SEXP scale_);
 
 // standardize a matrix of in-memory data
 extern SEXP in_mem_std(SEXP X_);
@@ -21,8 +24,8 @@ extern SEXP big_crossprod(SEXP X_,
                           SEXP ncore_);
 
 static const R_CallMethodDef callMethods[] = {
-  {"big_crossprod", (DL_FUNC) &big_crossprod, 4},
-  {"big_std", (DL_FUNC) &big_std, 2},
+  {"big_crossprod", (DL_FUNC) &big_crossprod, 3},
+  {"big_std", (DL_FUNC) &big_std, 4},
   {"big_sd", (DL_FUNC) &big_sd, 2},
   {"in_mem_std", (DL_FUNC) &in_mem_std, 1},
   {NULL, NULL, 0}
